@@ -1,13 +1,13 @@
 /**
-  @file src/base/omx_base_source.c
+  src/base/omx_base_source.c
 
   OpenMAX base source component. This component does not perform any multimedia
   processing. It derives from base component and contains a single output port.
   This class can also be used for a two port source component
   It can be used as base class for source components.
 
-  Copyright (C) 2007-2008 STMicroelectronics
-  Copyright (C) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
+  Copyright (C) 2007-2009 STMicroelectronics
+  Copyright (C) 2007-2009 Nokia Corporation and/or its subsidiary(-ies).
 
   This library is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the Free
@@ -24,9 +24,6 @@
   51 Franklin St, Fifth Floor, Boston, MA
   02110-1301  USA
 
-  $Date$
-  Revision $Rev$
-  Author $Author$
 */
 
 #include <omxcore.h>
@@ -34,7 +31,7 @@
 
 OMX_ERRORTYPE omx_base_source_Constructor(OMX_COMPONENTTYPE *openmaxStandComp, OMX_STRING cComponentName) {
 
-  OMX_ERRORTYPE err = OMX_ErrorNone;  
+  OMX_ERRORTYPE err = OMX_ErrorNone;
   omx_base_source_PrivateType* omx_base_source_Private;
 
   if (openmaxStandComp->pComponentPrivate) {
@@ -47,7 +44,7 @@ OMX_ERRORTYPE omx_base_source_Constructor(OMX_COMPONENTTYPE *openmaxStandComp, O
   }
 
   // we could create our own port structures here
-  // fixme maybe the base class could use a "port factory" function pointer?  
+  // fixme maybe the base class could use a "port factory" function pointer?
   err = omx_base_component_Constructor(openmaxStandComp, cComponentName);
 
   /* here we can override whatever defaults the base_component constructor set
@@ -64,7 +61,7 @@ OMX_ERRORTYPE omx_base_source_Destructor(OMX_COMPONENTTYPE *openmaxStandComp) {
 }
 
 /** This is the central function for component processing. It
-  * is executed in a separate thread, is synchronized with 
+  * is executed in a separate thread, is synchronized with
   * semaphores at each port, those are released each time a new buffer
   * is available on the given port.
   */
@@ -81,8 +78,11 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
   OMX_BOOL isOutputBufferNeeded = OMX_TRUE;
   int outBufExchanged = 0;
 
+  omx_base_source_Private->bellagioThreads->nThreadBufferMngtID = (long int)syscall(__NR_gettid);
+  DEBUG(DEB_LEV_SIMPLE_SEQ, "In %s the thread ID is %i\n", __func__, (int)omx_base_source_Private->bellagioThreads->nThreadBufferMngtID);
+
   DEBUG(DEB_LEV_FUNCTION_NAME, "In %s \n", __func__);
-  while(omx_base_component_Private->state == OMX_StateIdle || omx_base_component_Private->state == OMX_StateExecuting ||  
+  while(omx_base_component_Private->state == OMX_StateIdle || omx_base_component_Private->state == OMX_StateExecuting ||
     omx_base_component_Private->state == OMX_StatePause || omx_base_component_Private->transientState == OMX_TransStateLoadedToIdle){
 
     /*Wait till the ports are being flushed*/
@@ -98,7 +98,7 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
         DEBUG(DEB_LEV_FULL_SEQ, "Ports are flushing,so returning output buffer\n");
       }
       DEBUG(DEB_LEV_FULL_SEQ, "In %s signalling flush all condition \n", __func__);
-      
+
       tsem_up(omx_base_source_Private->flush_all_condition);
       tsem_down(omx_base_source_Private->flush_condition);
       pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
@@ -106,7 +106,7 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
     pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
 
     /*No buffer to process. So wait here*/
-    if((isOutputBufferNeeded==OMX_TRUE && pOutputSem->semval==0) && 
+    if((isOutputBufferNeeded==OMX_TRUE && pOutputSem->semval==0) &&
       (omx_base_source_Private->state != OMX_StateLoaded && omx_base_source_Private->state != OMX_StateInvalid)) {
       DEBUG(DEB_LEV_SIMPLE_SEQ, "Waiting for output buffer \n");
       tsem_down(omx_base_source_Private->bMgmtSem);
@@ -130,21 +130,19 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
         }
       }
     }
-    
+
     if(isOutputBufferNeeded == OMX_FALSE) {
-      if(pOutputBuffer->nFlags == OMX_BUFFERFLAG_EOS) {
-        DEBUG(DEB_LEV_SIMPLE_SEQ, "Detected EOS flags in output buffer\n");
-        
-        (*(omx_base_component_Private->callbacks->EventHandler))
-          (openmaxStandComp,
-          omx_base_component_Private->callbackData,
-          OMX_EventBufferFlag, /* The command was completed */
-          0, /* The commands was a OMX_CommandStateSet */
-          pOutputBuffer->nFlags, /* The state has been changed in message->messageParam2 */
-          NULL);
+      if((pOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS) {
         pOutputBuffer->nFlags = 0;
       }
-      
+
+      if(omx_base_source_Private->pMark.hMarkTargetComponent != NULL){
+        pOutputBuffer->hMarkTargetComponent = omx_base_source_Private->pMark.hMarkTargetComponent;
+        pOutputBuffer->pMarkData            = omx_base_source_Private->pMark.pMarkData;
+        omx_base_source_Private->pMark.hMarkTargetComponent = NULL;
+        omx_base_source_Private->pMark.pMarkData            = NULL;
+      }
+
       target_component = (OMX_COMPONENTTYPE*)pOutputBuffer->hMarkTargetComponent;
       if(target_component == (OMX_COMPONENTTYPE *)openmaxStandComp) {
         /*Clear the mark and generate an event*/
@@ -157,28 +155,47 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
           pOutputBuffer->pMarkData);
       } else if(pOutputBuffer->hMarkTargetComponent != NULL) {
         /*If this is not the target component then pass the mark*/
-        DEBUG(DEB_LEV_FULL_SEQ, "Can't Pass Mark. This is a Source!!\n");
+        DEBUG(DEB_LEV_FULL_SEQ, "Pass Mark. This is a Source!!\n");
       }
 
-      if (omx_base_source_Private->BufferMgmtCallback && pOutputBuffer->nFilledLen == 0) {
-        (*(omx_base_source_Private->BufferMgmtCallback))(openmaxStandComp, pOutputBuffer);
+      if(omx_base_source_Private->state == OMX_StateExecuting)  {
+        if (omx_base_source_Private->BufferMgmtCallback && pOutputBuffer->nFilledLen == 0) {
+          (*(omx_base_source_Private->BufferMgmtCallback))(openmaxStandComp, pOutputBuffer);
+        } else {
+          /*It no buffer management call back then don't produce any output buffer*/
+          pOutputBuffer->nFilledLen = 0;
+        }
       } else {
-        /*It no buffer management call back then don't produce any output buffer*/
-        pOutputBuffer->nFilledLen = 0;
+        DEBUG(DEB_LEV_ERR, "In %s Received Buffer in non-Executing State(%x)\n", __func__, (int)omx_base_source_Private->state);
       }
       if(omx_base_source_Private->state == OMX_StatePause && !PORT_IS_BEING_FLUSHED(pOutPort)) {
         /*Waiting at paused state*/
         tsem_wait(omx_base_source_Private->bStateSem);
       }
 
-      /*Output Buffer has been produced or EOS. So, return output buffer and get new buffer*/
-      if(pOutputBuffer->nFilledLen != 0 || pOutputBuffer->nFlags == OMX_BUFFERFLAG_EOS) {
-        pOutPort->ReturnBufferFunction(pOutPort, pOutputBuffer);
-        outBufExchanged--;
-        pOutputBuffer = NULL;
-        isOutputBufferNeeded = OMX_TRUE;
+      if((pOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS) {
+        DEBUG(DEB_LEV_SIMPLE_SEQ, "Detected EOS flags in output buffer\n");
+
+        (*(omx_base_component_Private->callbacks->EventHandler))
+          (openmaxStandComp,
+          omx_base_component_Private->callbackData,
+          OMX_EventBufferFlag, /* The command was completed */
+          0, /* The commands was a OMX_CommandStateSet */
+          pOutputBuffer->nFlags, /* The state has been changed in message->messageParam2 */
+          NULL);
+        //pOutputBuffer->nFlags = 0;
+        omx_base_source_Private->bIsEOSReached = OMX_TRUE;
       }
 
+      /*Output Buffer has been produced or EOS. So, return output buffer and get new buffer*/
+      if((pOutputBuffer->nFilledLen != 0) || ((pOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) ==OMX_BUFFERFLAG_EOS) || (omx_base_source_Private->bIsEOSReached == OMX_TRUE)) {
+    	  if((pOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS)
+    		  DEBUG(DEB_LEV_SIMPLE_SEQ, "In %s nFlags=%x Name=%s \n", __func__, (int)pOutputBuffer->nFlags, omx_base_source_Private->name);
+    	  pOutPort->ReturnBufferFunction(pOutPort, pOutputBuffer);
+    	  outBufExchanged--;
+    	  pOutputBuffer = NULL;
+    	  isOutputBufferNeeded = OMX_TRUE;
+      }
     }
   }
   DEBUG(DEB_LEV_SIMPLE_SEQ, "Exiting Buffer Management Thread\n");
@@ -186,7 +203,7 @@ void* omx_base_source_BufferMgmtFunction (void* param) {
 }
 
 /** This is the central function for buffer processing of a two port source component.
-  * It is executed in a separate thread, is synchronized with 
+  * It is executed in a separate thread, is synchronized with
   * semaphores at each port, those are released each time a new buffer
   * is available on the given port.
   */
@@ -214,16 +231,16 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
   outBufExchanged[0]=outBufExchanged[1]=0;
 
   DEBUG(DEB_LEV_FUNCTION_NAME, "In %s\n", __func__);
-  while(omx_base_source_Private->state == OMX_StateIdle || omx_base_source_Private->state == OMX_StateExecuting ||  omx_base_source_Private->state == OMX_StatePause || 
+  while(omx_base_source_Private->state == OMX_StateIdle || omx_base_source_Private->state == OMX_StateExecuting ||  omx_base_source_Private->state == OMX_StatePause ||
     omx_base_source_Private->transientState == OMX_TransStateLoadedToIdle){
 
     /*Wait till the ports are being flushed*/
     pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
-    while( PORT_IS_BEING_FLUSHED(pOutPort[0]) || 
+    while( PORT_IS_BEING_FLUSHED(pOutPort[0]) ||
            PORT_IS_BEING_FLUSHED(pOutPort[1])) {
       pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
-      
-      DEBUG(DEB_LEV_FULL_SEQ, "In %s 1 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n", 
+
+      DEBUG(DEB_LEV_FULL_SEQ, "In %s 1 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n",
         __func__,outBufExchanged[0],isOutputBufferNeeded[0],outBufExchanged[1],isOutputBufferNeeded[1],pOutputSem[0]->semval,pOutputSem[1]->semval);
 
       if(isOutputBufferNeeded[1]==OMX_FALSE && PORT_IS_BEING_FLUSHED(pOutPort[1])) {
@@ -242,9 +259,9 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
         DEBUG(DEB_LEV_FULL_SEQ, "Ports are flushing,so returning output 0 buffer\n");
       }
 
-      DEBUG(DEB_LEV_FULL_SEQ, "In %s 2 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n", 
+      DEBUG(DEB_LEV_FULL_SEQ, "In %s 2 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n",
         __func__,outBufExchanged[0],isOutputBufferNeeded[0],outBufExchanged[1],isOutputBufferNeeded[1],pOutputSem[0]->semval,pOutputSem[1]->semval);
-  
+
       tsem_up(omx_base_source_Private->flush_all_condition);
       tsem_down(omx_base_source_Private->flush_condition);
       pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
@@ -252,30 +269,30 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
     pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
 
     /*No buffer to process. So wait here*/
-    if((isOutputBufferNeeded[0]==OMX_TRUE && pOutputSem[0]->semval==0) && 
+    if((isOutputBufferNeeded[0]==OMX_TRUE && pOutputSem[0]->semval==0) &&
       (omx_base_source_Private->state != OMX_StateLoaded && omx_base_source_Private->state != OMX_StateInvalid)) {
       //Signalled from EmptyThisBuffer or FillThisBuffer or some thing else
       DEBUG(DEB_LEV_FULL_SEQ, "Waiting for next output buffer 0\n");
       tsem_down(omx_base_source_Private->bMgmtSem);
-      
+
     }
     if(omx_base_source_Private->state == OMX_StateLoaded || omx_base_source_Private->state == OMX_StateInvalid) {
       DEBUG(DEB_LEV_SIMPLE_SEQ, "In %s Buffer Management Thread is exiting\n",__func__);
       break;
     }
-    if((isOutputBufferNeeded[1]==OMX_TRUE && pOutputSem[1]->semval==0) && 
+    if((isOutputBufferNeeded[1]==OMX_TRUE && pOutputSem[1]->semval==0) &&
       (omx_base_source_Private->state != OMX_StateLoaded && omx_base_source_Private->state != OMX_StateInvalid) &&
        !(PORT_IS_BEING_FLUSHED(pOutPort[0]) || PORT_IS_BEING_FLUSHED(pOutPort[1]))) {
       //Signalled from EmptyThisBuffer or FillThisBuffer or some thing else
       DEBUG(DEB_LEV_FULL_SEQ, "Waiting for next output buffer 1\n");
       tsem_down(omx_base_source_Private->bMgmtSem);
-      
+
     }
     if(omx_base_source_Private->state == OMX_StateLoaded || omx_base_source_Private->state == OMX_StateInvalid) {
       DEBUG(DEB_LEV_SIMPLE_SEQ, "In %s Buffer Management Thread is exiting\n",__func__);
       break;
     }
- 
+
     DEBUG(DEB_LEV_SIMPLE_SEQ, "Waiting for output buffer 0 semval=%d \n",pOutputSem[0]->semval);
     if(pOutputSem[0]->semval>0 && isOutputBufferNeeded[0]==OMX_TRUE ) {
       tsem_down(pOutputSem[0]);
@@ -311,12 +328,11 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
       if(omx_base_source_Private->ports[i]->sPortParam.eDomain!=OMX_PortDomainOther){ /* clock ports are not to be processed */
         /*Process Output buffer of Port i */
         if(isOutputBufferNeeded[i]==OMX_FALSE) {
-          
+
+          /*Pass the Mark to all outgoing buffers*/
           if(omx_base_source_Private->pMark.hMarkTargetComponent != NULL){
             pOutputBuffer[i]->hMarkTargetComponent = omx_base_source_Private->pMark.hMarkTargetComponent;
             pOutputBuffer[i]->pMarkData            = omx_base_source_Private->pMark.pMarkData;
-            omx_base_source_Private->pMark.hMarkTargetComponent = NULL;
-            omx_base_source_Private->pMark.pMarkData            = NULL;
           }
 
           target_component=(OMX_COMPONENTTYPE*)pOutputBuffer[i]->hMarkTargetComponent;
@@ -331,18 +347,21 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
               pOutputBuffer[i]->pMarkData);
           } else if(pOutputBuffer[i]->hMarkTargetComponent!=NULL){
             /*If this is not the target component then pass the mark*/
-            pOutputBuffer[i]->pMarkData=NULL;
+            DEBUG(DEB_LEV_FULL_SEQ, "Pass Mark. This is a Source!!\n");
           }
 
-          if (omx_base_source_Private->BufferMgmtCallback && pOutputBuffer[i]->nFilledLen == 0) {
-            //(*(omx_base_source_Private->BufferMgmtCallback))(openmaxStandComp, pOutputBuffer[0], pOutputBuffer[1]);
-            (*(omx_base_source_Private->BufferMgmtCallback))(openmaxStandComp, pOutputBuffer[i]);
+          if(omx_base_source_Private->state == OMX_StateExecuting)  {
+            if (omx_base_source_Private->BufferMgmtCallback && pOutputBuffer[i]->nFilledLen == 0) {
+              (*(omx_base_source_Private->BufferMgmtCallback))(openmaxStandComp, pOutputBuffer[i]);
+            } else {
+              /*If no buffer management call back then don't produce any output buffer*/
+              pOutputBuffer[i]->nFilledLen = 0;
+            }
           } else {
-            /*If no buffer management call back then don't produce any output buffer*/
-            pOutputBuffer[i]->nFilledLen = 0;
+            DEBUG(DEB_LEV_ERR, "In %s Received Buffer in non-Executing State(%x)\n", __func__, (int)omx_base_source_Private->state);
           }
-      
-          if(pOutputBuffer[i]->nFlags==OMX_BUFFERFLAG_EOS && pOutputBuffer[i]->nFilledLen==0) {
+
+          if((pOutputBuffer[i]->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS && pOutputBuffer[i]->nFilledLen==0) {
             DEBUG(DEB_LEV_FULL_SEQ, "Detected EOS flags in input buffer filled len=%d\n", (int)pOutputBuffer[i]->nFilledLen);
             (*(omx_base_source_Private->callbacks->EventHandler))
               (openmaxStandComp,
@@ -358,7 +377,7 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
           }
 
            /*Output Buffer has been produced or EOS. So, return output buffer and get new buffer*/
-          if(pOutputBuffer[i]->nFilledLen!=0 || pOutputBuffer[i]->nFlags==OMX_BUFFERFLAG_EOS){
+          if(pOutputBuffer[i]->nFilledLen!=0 || (pOutputBuffer[i]->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS){
             pOutPort[i]->ReturnBufferFunction(pOutPort[i],pOutputBuffer[i]);
             outBufExchanged[i]--;
             pOutputBuffer[i]=NULL;
@@ -367,6 +386,12 @@ void* omx_base_source_twoport_BufferMgmtFunction (void* param) {
         }
       }
     }
+
+    /*Clear the Mark*/
+    if(omx_base_source_Private->pMark.hMarkTargetComponent != NULL){
+      omx_base_source_Private->pMark.hMarkTargetComponent = NULL;
+      omx_base_source_Private->pMark.pMarkData            = NULL;
+    }
   }
   DEBUG(DEB_LEV_SIMPLE_SEQ,"Exiting Buffer Management Thread\n");
   return NULL;
