@@ -1,11 +1,11 @@
 /**
-  @file src/components/audio_effects/omx_audiomixer_component.c
+  src/components/audio_effects/omx_audiomixer_component.c
 
-  OpenMAX audio mixer control component. This component implements a mixer that 
+  OpenMAX audio mixer control component. This component implements a mixer that
   mixes multiple audio PCM streams and produces a single output stream.
 
-  Copyright (C) 2008  STMicroelectronics
-  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
+  Copyright (C) 2008-2009 STMicroelectronics
+  Copyright (C) 2008-2009 Nokia Corporation and/or its subsidiary(-ies).
 
   This library is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the Free
@@ -22,9 +22,6 @@
   51 Franklin St, Fifth Floor, Boston, MA
   02110-1301  USA
 
-  $Date$
-  Revision $Rev$
-  Author $Author$
 */
 
 #include <omxcore.h>
@@ -32,26 +29,16 @@
 #include <omx_audiomixer_component.h>
 #include<OMX_Audio.h>
 
-#define OMX_AUDIO_MIXER_INPUTPORT_INDEX      0
-#define OMX_AUDIO_MIXER_INPUTPORT_INDEX_1    1
-#define OMX_AUDIO_MIXER_OUTPUTPORT_INDEX     2
-
 /* Gain value */
 #define GAIN_VALUE 100.0f
 
-/* Max allowable audio_mixer component instance */
-#define MAX_COMPONENT_AUDIO_MIXER 1
-
-/** Maximum Number of AudioMixer Component Instance*/
-static OMX_U32 noAudioMixerCompInstance = 0;
-
-
 OMX_ERRORTYPE omx_audio_mixer_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp, OMX_STRING cComponentName) {
-  OMX_ERRORTYPE err = OMX_ErrorNone;  
+  OMX_ERRORTYPE err = OMX_ErrorNone;
   omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private;
   omx_audio_mixer_component_PortType *pPort;//,*inPort1, *outPort;
   OMX_U32 i;
 
+  RM_RegisterComponent(MIXER_COMP_NAME, MAX_MIXER_COMPONENTS);
   if (!openmaxStandComp->pComponentPrivate) {
     DEBUG(DEB_LEV_FUNCTION_NAME, "In %s, allocating component\n",__func__);
     openmaxStandComp->pComponentPrivate = calloc(1, sizeof(omx_audio_mixer_component_PrivateType));
@@ -59,19 +46,20 @@ OMX_ERRORTYPE omx_audio_mixer_component_Constructor(OMX_COMPONENTTYPE *openmaxSt
       return OMX_ErrorInsufficientResources;
     }
   } else {
-    DEBUG(DEB_LEV_FUNCTION_NAME, "In %s, Error Component %x Already Allocated\n", __func__, (int)openmaxStandComp->pComponentPrivate);
+    DEBUG(DEB_LEV_FUNCTION_NAME, "In %s, Error Component %p Already Allocated\n", __func__, openmaxStandComp->pComponentPrivate);
   }
 
   omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
   omx_audio_mixer_component_Private->ports = NULL;
-  
+
   /** Calling base filter constructor */
   err = omx_base_filter_Constructor(openmaxStandComp, cComponentName);
 
-  /*Assuming 2 input and 1 output portd*/
+  /*Assuming 4 input and 1 output ports*/
+  omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nStartPortNumber = 0;
   omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts = MAX_PORTS;
 
-  /** Allocate Ports and call port constructor. */  
+  /** Allocate Ports and call port constructor. */
   if (omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts && !omx_audio_mixer_component_Private->ports) {
     omx_audio_mixer_component_Private->ports = calloc(omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts, sizeof(omx_base_PortType *));
     if (!omx_audio_mixer_component_Private->ports) {
@@ -89,14 +77,14 @@ OMX_ERRORTYPE omx_audio_mixer_component_Constructor(OMX_COMPONENTTYPE *openmaxSt
   for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++) {
     base_audio_port_Constructor(openmaxStandComp, &omx_audio_mixer_component_Private->ports[i], i, OMX_TRUE);
   }
-  
+
   /* construct one output port */
   base_audio_port_Constructor(openmaxStandComp, &omx_audio_mixer_component_Private->ports[omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1], omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1, OMX_FALSE);
-  
-  /** Domain specific section for the ports. */  
+
+  /** Domain specific section for the ports. */
   for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++) {
     pPort = (omx_audio_mixer_component_PortType *) omx_audio_mixer_component_Private->ports[i];
-    
+
     pPort->sPortParam.nBufferSize = DEFAULT_OUT_BUFFER_SIZE;
     pPort->gain = GAIN_VALUE; //100.0f; // default gain
 
@@ -112,7 +100,7 @@ OMX_ERRORTYPE omx_audio_mixer_component_Constructor(OMX_COMPONENTTYPE *openmaxSt
 
     setHeader(&pPort->sVolume,sizeof(OMX_AUDIO_CONFIG_VOLUMETYPE));
     pPort->sVolume.nPortIndex = i;
-    pPort->sVolume.bLinear = OMX_TRUE;           /**< Is the volume to be set in linear (0.100) 
+    pPort->sVolume.bLinear = OMX_TRUE;           /**< Is the volume to be set in linear (0.100)
                                      or logarithmic scale (mB) */
     pPort->sVolume.sVolume.nValue = (OMX_S32)GAIN_VALUE;
     pPort->sVolume.sVolume.nMin = 0;   /**< minimum for value (i.e. nValue >= nMin) */
@@ -127,9 +115,14 @@ OMX_ERRORTYPE omx_audio_mixer_component_Constructor(OMX_COMPONENTTYPE *openmaxSt
   omx_audio_mixer_component_Private->BufferMgmtCallback = omx_audio_mixer_component_BufferMgmtCallback;
   omx_audio_mixer_component_Private->BufferMgmtFunction = omx_audio_mixer_BufferMgmtFunction;
 
-  noAudioMixerCompInstance++;
-  if(noAudioMixerCompInstance > MAX_COMPONENT_AUDIO_MIXER) {
-    return OMX_ErrorInsufficientResources;
+  /* resource management special section */
+  omx_audio_mixer_component_Private->nqualitylevels = MIXER_QUALITY_LEVELS;
+  omx_audio_mixer_component_Private->currentQualityLevel = 1;
+  omx_audio_mixer_component_Private->multiResourceLevel = malloc(sizeof(multiResourceDescriptor *) * MIXER_QUALITY_LEVELS);
+  for (i = 0; i<MIXER_QUALITY_LEVELS; i++) {
+	  omx_audio_mixer_component_Private->multiResourceLevel[i] = malloc(sizeof(multiResourceDescriptor));
+	  omx_audio_mixer_component_Private->multiResourceLevel[i]->CPUResourceRequested = mixerQualityLevels[i * 2];
+	  omx_audio_mixer_component_Private->multiResourceLevel[i]->MemoryResourceRequested = mixerQualityLevels[i * 2 + 1];
   }
 
   return err;
@@ -143,6 +136,7 @@ OMX_ERRORTYPE omx_audio_mixer_component_Destructor(OMX_COMPONENTTYPE *openmaxSta
   omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
   OMX_U32 i;
 
+  DEBUG(DEB_LEV_FUNCTION_NAME, "In %s\n", __func__);
   /* frees port/s */
   if (omx_audio_mixer_component_Private->ports) {
     for (i=0; i < omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts; i++) {
@@ -153,9 +147,9 @@ OMX_ERRORTYPE omx_audio_mixer_component_Destructor(OMX_COMPONENTTYPE *openmaxSta
     omx_audio_mixer_component_Private->ports=NULL;
   }
 
-  DEBUG(DEB_LEV_FUNCTION_NAME, "Destructor of audiodecoder component is called\n");
   omx_base_filter_Destructor(openmaxStandComp);
-  noAudioMixerCompInstance--;
+
+  DEBUG(DEB_LEV_FUNCTION_NAME, "Out of %s\n", __func__);
 
   return OMX_ErrorNone;
 }
@@ -167,7 +161,7 @@ void omx_audio_mixer_component_BufferMgmtCallback(OMX_COMPONENTTYPE *openmaxStan
   OMX_U32 i,sampleCount = pInBuffer->nFilledLen / 2; // signed 16 bit samples assumed
   omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
   omx_audio_mixer_component_PortType* pPort;
-  
+
   for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++) {
     pPort = (omx_audio_mixer_component_PortType*)omx_audio_mixer_component_Private->ports[i];
     if(PORT_IS_ENABLED(pPort)){
@@ -198,9 +192,9 @@ void omx_audio_mixer_component_BufferMgmtCallback(OMX_COMPONENTTYPE *openmaxStan
 
 /** setting configurations */
 OMX_ERRORTYPE omx_audio_mixer_component_SetConfig(
-  OMX_IN  OMX_HANDLETYPE hComponent,
-  OMX_IN  OMX_INDEXTYPE nIndex,
-  OMX_IN  OMX_PTR pComponentConfigStructure) {
+  OMX_HANDLETYPE hComponent,
+  OMX_INDEXTYPE nIndex,
+  OMX_PTR pComponentConfigStructure) {
 
   OMX_AUDIO_CONFIG_VOLUMETYPE* pVolume;
   OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
@@ -209,13 +203,13 @@ OMX_ERRORTYPE omx_audio_mixer_component_SetConfig(
   OMX_ERRORTYPE err = OMX_ErrorNone;
 
   switch (nIndex) {
-    case OMX_IndexConfigAudioVolume : 
+    case OMX_IndexConfigAudioVolume :
       pVolume = (OMX_AUDIO_CONFIG_VOLUMETYPE*) pComponentConfigStructure;
       if(pVolume->sVolume.nValue > 100) {
         err =  OMX_ErrorBadParameter;
         break;
       }
-      
+
       if (pVolume->nPortIndex <= omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts) {
         pPort= (omx_audio_mixer_component_PortType *)omx_audio_mixer_component_Private->ports[pVolume->nPortIndex];
         DEBUG(DEB_LEV_SIMPLE_SEQ, "Port %i Gain=%d\n",(int)pVolume->nPortIndex,(int)pVolume->sVolume.nValue);
@@ -231,17 +225,17 @@ OMX_ERRORTYPE omx_audio_mixer_component_SetConfig(
 }
 
 OMX_ERRORTYPE omx_audio_mixer_component_GetConfig(
-  OMX_IN  OMX_HANDLETYPE hComponent,
-  OMX_IN  OMX_INDEXTYPE nIndex,
-  OMX_INOUT OMX_PTR pComponentConfigStructure) {
-  OMX_AUDIO_CONFIG_VOLUMETYPE* pVolume;
+  OMX_HANDLETYPE hComponent,
+  OMX_INDEXTYPE nIndex,
+  OMX_PTR pComponentConfigStructure) {
+  OMX_AUDIO_CONFIG_VOLUMETYPE           *pVolume;
   OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
-  omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
-  omx_audio_mixer_component_PortType * pPort;
+  omx_audio_mixer_component_PrivateType *omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
+  omx_audio_mixer_component_PortType    *pPort;
   OMX_ERRORTYPE err = OMX_ErrorNone;
 
   switch (nIndex) {
-    case OMX_IndexConfigAudioVolume : 
+    case OMX_IndexConfigAudioVolume :
       pVolume = (OMX_AUDIO_CONFIG_VOLUMETYPE*) pComponentConfigStructure;
       if (pVolume->nPortIndex <= omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts) {
         pPort= (omx_audio_mixer_component_PortType *)omx_audio_mixer_component_Private->ports[pVolume->nPortIndex];
@@ -257,13 +251,14 @@ OMX_ERRORTYPE omx_audio_mixer_component_GetConfig(
 }
 
 OMX_ERRORTYPE omx_audio_mixer_component_SetParameter(
-  OMX_IN  OMX_HANDLETYPE hComponent,
-  OMX_IN  OMX_INDEXTYPE nParamIndex,
-  OMX_IN  OMX_PTR ComponentParameterStructure) {
+  OMX_HANDLETYPE hComponent,
+  OMX_INDEXTYPE nParamIndex,
+  OMX_PTR ComponentParameterStructure) {
 
-  OMX_ERRORTYPE err = OMX_ErrorNone;
-  OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
-  OMX_U32 portIndex;
+  OMX_ERRORTYPE                   err = OMX_ErrorNone;
+  OMX_AUDIO_PARAM_PORTFORMATTYPE  *pAudioPortFormat;
+  OMX_PARAM_COMPONENTROLETYPE     *pComponentRole;
+  OMX_U32                         portIndex;
   omx_audio_mixer_component_PortType *port;
 
   /* Check which structure we are being fed and make control its header */
@@ -279,8 +274,8 @@ OMX_ERRORTYPE omx_audio_mixer_component_SetParameter(
       pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
       portIndex = pAudioPortFormat->nPortIndex;
       err = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pAudioPortFormat, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
-      if(err!=OMX_ErrorNone) { 
-        DEBUG(DEB_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err); 
+      if(err!=OMX_ErrorNone) {
+        DEBUG(DEB_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err);
         break;
       }
       if (portIndex <= omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts) {
@@ -289,7 +284,23 @@ OMX_ERRORTYPE omx_audio_mixer_component_SetParameter(
       } else {
         err = OMX_ErrorBadPortIndex;
       }
-      break;  
+      break;
+    case OMX_IndexParamStandardComponentRole:
+      pComponentRole = (OMX_PARAM_COMPONENTROLETYPE*)ComponentParameterStructure;
+
+      if (omx_audio_mixer_component_Private->state != OMX_StateLoaded && omx_audio_mixer_component_Private->state != OMX_StateWaitForResources) {
+        DEBUG(DEB_LEV_ERR, "In %s Incorrect State=%x lineno=%d\n",__func__,omx_audio_mixer_component_Private->state,__LINE__);
+        return OMX_ErrorIncorrectStateOperation;
+      }
+
+      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PARAM_COMPONENTROLETYPE))) != OMX_ErrorNone) {
+        break;
+      }
+
+      if (strcmp( (char*) pComponentRole->cRole, MIXER_COMP_ROLE)) {
+        return OMX_ErrorBadParameter;
+      }
+      break;
     default:
       err = omx_base_component_SetParameter(hComponent, nParamIndex, ComponentParameterStructure);
   }
@@ -297,16 +308,17 @@ OMX_ERRORTYPE omx_audio_mixer_component_SetParameter(
 }
 
 OMX_ERRORTYPE omx_audio_mixer_component_GetParameter(
-  OMX_IN  OMX_HANDLETYPE hComponent,
-  OMX_IN  OMX_INDEXTYPE nParamIndex,
-  OMX_INOUT OMX_PTR ComponentParameterStructure) {
+  OMX_HANDLETYPE hComponent,
+  OMX_INDEXTYPE nParamIndex,
+  OMX_PTR ComponentParameterStructure) {
 
-  OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;  
-  OMX_AUDIO_PARAM_PCMMODETYPE *pAudioPcmMode;
-  OMX_ERRORTYPE err = OMX_ErrorNone;
-  omx_audio_mixer_component_PortType *port;
-  OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
-  omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
+  OMX_AUDIO_PARAM_PORTFORMATTYPE  *pAudioPortFormat;
+  OMX_AUDIO_PARAM_PCMMODETYPE     *pAudioPcmMode;
+  OMX_PARAM_COMPONENTROLETYPE     *pComponentRole;
+  OMX_ERRORTYPE                   err = OMX_ErrorNone;
+  omx_audio_mixer_component_PortType    *port;
+  OMX_COMPONENTTYPE                     *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
+  omx_audio_mixer_component_PrivateType *omx_audio_mixer_component_Private = openmaxStandComp->pComponentPrivate;
   if (ComponentParameterStructure == NULL) {
     return OMX_ErrorBadParameter;
   }
@@ -314,51 +326,80 @@ OMX_ERRORTYPE omx_audio_mixer_component_GetParameter(
   /* Check which structure we are being fed and fill its header */
   switch(nParamIndex) {
     case OMX_IndexParamAudioInit:
-      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone) { 
+      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone) {
         break;
       }
       memcpy(ComponentParameterStructure, &omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio], sizeof(OMX_PORT_PARAM_TYPE));
-      break;    
+      break;
     case OMX_IndexParamAudioPortFormat:
       pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
-      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone) { 
+      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone) {
         break;
       }
-      if (pAudioPortFormat->nPortIndex <= 1) {
+      if (pAudioPortFormat->nPortIndex <= omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts) {
         port= (omx_audio_mixer_component_PortType *)omx_audio_mixer_component_Private->ports[pAudioPortFormat->nPortIndex];
         memcpy(pAudioPortFormat, &port->sAudioParam, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
       } else {
         err = OMX_ErrorBadPortIndex;
       }
-      break;    
+      break;
     case OMX_IndexParamAudioPcm:
       pAudioPcmMode = (OMX_AUDIO_PARAM_PCMMODETYPE*)ComponentParameterStructure;
-      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE))) != OMX_ErrorNone) { 
+      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE))) != OMX_ErrorNone) {
         break;
       }
 
       if (pAudioPcmMode->nPortIndex <= omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts) {
         port= (omx_audio_mixer_component_PortType *)omx_audio_mixer_component_Private->ports[pAudioPcmMode->nPortIndex];
-        memcpy(pAudioPcmMode, &port->pAudioPcmMode, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE)); 
+        memcpy(pAudioPcmMode, &port->pAudioPcmMode, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));
       } else {
         err = OMX_ErrorBadPortIndex;
       }
       break;
+    case OMX_IndexParamStandardComponentRole:
+      pComponentRole = (OMX_PARAM_COMPONENTROLETYPE*)ComponentParameterStructure;
+      if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PARAM_COMPONENTROLETYPE))) != OMX_ErrorNone) {
+        break;
+      }
+      strcpy( (char*) pComponentRole->cRole, MIXER_COMP_ROLE);
+      break;
     default:
       err = omx_base_component_GetParameter(hComponent, nParamIndex, ComponentParameterStructure);
   }
   return err;
 }
 
+int checkAnyPortBeingFlushed(omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private) {
+  omx_base_PortType *pPort;
+  int ret = OMX_FALSE,i;
+
+  if(omx_audio_mixer_component_Private->state == OMX_StateLoaded ||
+     omx_audio_mixer_component_Private->state == OMX_StateInvalid) {
+    return 0;
+  }
+
+  pthread_mutex_lock(&omx_audio_mixer_component_Private->flush_mutex);
+  for (i=0; i < omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts; i++) {
+    pPort = omx_audio_mixer_component_Private->ports[i];
+    if(PORT_IS_BEING_FLUSHED(pPort)) {
+      ret = OMX_TRUE;
+      break;
+    }
+  }
+  pthread_mutex_unlock(&omx_audio_mixer_component_Private->flush_mutex);
+
+  return ret;
+}
+
 /** This is the central function for component processing,overridden for audio mixer. It
-  * is executed in a separate thread, is synchronized with 
+  * is executed in a separate thread, is synchronized with
   * semaphores at each port, those are released each time a new buffer
   * is available on the given port.
   */
 void* omx_audio_mixer_BufferMgmtFunction (void* param) {
   OMX_COMPONENTTYPE* openmaxStandComp = (OMX_COMPONENTTYPE*)param;
   omx_audio_mixer_component_PrivateType* omx_audio_mixer_component_Private = (omx_audio_mixer_component_PrivateType*)openmaxStandComp->pComponentPrivate;
-  
+
   omx_base_PortType *pPort[MAX_PORTS];
   tsem_t* pSem[MAX_PORTS];
   queue_t* pQueue[MAX_PORTS];
@@ -379,19 +420,13 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
 
 
   DEBUG(DEB_LEV_FUNCTION_NAME, "In %s\n", __func__);
-  while(omx_audio_mixer_component_Private->state == OMX_StateIdle || omx_audio_mixer_component_Private->state == OMX_StateExecuting ||  omx_audio_mixer_component_Private->state == OMX_StatePause || 
-    omx_audio_mixer_component_Private->transientState == OMX_TransStateLoadedToIdle){
+  while(omx_audio_mixer_component_Private->state == OMX_StateIdle || omx_audio_mixer_component_Private->state == OMX_StateExecuting ||  omx_audio_mixer_component_Private->state == OMX_StatePause ||
+    omx_audio_mixer_component_Private->transientState == OMX_TransStateLoadedToIdle) {
 
     /*Wait till the ports are being flushed*/
-    pthread_mutex_lock(&omx_audio_mixer_component_Private->flush_mutex);
-    while( PORT_IS_BEING_FLUSHED(pPort[0]) || 
-           PORT_IS_BEING_FLUSHED(pPort[1]) || 
-           PORT_IS_BEING_FLUSHED(pPort[2]) || 
-           PORT_IS_BEING_FLUSHED(pPort[3]) || 
-           PORT_IS_BEING_FLUSHED(pPort[nOutputPortIndex])) {
-      pthread_mutex_unlock(&omx_audio_mixer_component_Private->flush_mutex);
-      
-      DEBUG(DEB_LEV_FULL_SEQ, "In %s 1 signalling flush all cond iF=%d,oF=%d iSemVal=%d,oSemval=%d\n", 
+    while( checkAnyPortBeingFlushed(omx_audio_mixer_component_Private) ) {
+
+      DEBUG(DEB_LEV_FULL_SEQ, "In %s 1 signalling flush all cond iF=%d,oF=%d iSemVal=%d,oSemval=%d\n",
         __func__,isBufferNeeded[0],isBufferNeeded[nOutputPortIndex],pSem[0]->semval,pSem[nOutputPortIndex]->semval);
 
       for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++){
@@ -403,31 +438,30 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
         }
       }
 
-      DEBUG(DEB_LEV_FULL_SEQ, "In %s 2 signalling flush all cond iF=%d,oF=%d iSemVal=%d,oSemval=%d\n", 
+      DEBUG(DEB_LEV_FULL_SEQ, "In %s 2 signalling flush all cond iF=%d,oF=%d iSemVal=%d,oSemval=%d\n",
         __func__,isBufferNeeded[0],isBufferNeeded[nOutputPortIndex],pSem[0]->semval,pSem[nOutputPortIndex]->semval);
-  
+
       tsem_up(omx_audio_mixer_component_Private->flush_all_condition);
       tsem_down(omx_audio_mixer_component_Private->flush_condition);
-      pthread_mutex_lock(&omx_audio_mixer_component_Private->flush_mutex);
     }
-    pthread_mutex_unlock(&omx_audio_mixer_component_Private->flush_mutex);
+
+    if(omx_audio_mixer_component_Private->state == OMX_StateLoaded || omx_audio_mixer_component_Private->state == OMX_StateInvalid) {
+      DEBUG(DEB_LEV_SIMPLE_SEQ, "In %s Buffer Management Thread is exiting\n",__func__);
+      break;
+    }
 
     /*No buffer to process. So wait here*/
-    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++){ 
-      if((isBufferNeeded[i]==OMX_TRUE && pSem[i]->semval==0) && 
+    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++){
+      if((isBufferNeeded[i]==OMX_TRUE && pSem[i]->semval==0) &&
         (omx_audio_mixer_component_Private->state != OMX_StateLoaded && omx_audio_mixer_component_Private->state != OMX_StateInvalid) &&
         PORT_IS_ENABLED(pPort[i]) && !PORT_IS_BEING_FLUSHED(pPort[i])) {
         //Signalled from EmptyThisBuffer or FillThisBuffer or some thing else
         DEBUG(DEB_LEV_FULL_SEQ, "Waiting for next input/output buffer\n");
         tsem_down(omx_audio_mixer_component_Private->bMgmtSem);
-      
+
       }
       /*Don't wait for buffers, if any port is flushing*/
-      if(PORT_IS_BEING_FLUSHED(pPort[0]) || 
-         PORT_IS_BEING_FLUSHED(pPort[1]) || 
-         PORT_IS_BEING_FLUSHED(pPort[2]) || 
-         PORT_IS_BEING_FLUSHED(pPort[3]) || 
-         PORT_IS_BEING_FLUSHED(pPort[nOutputPortIndex])){
+      if(checkAnyPortBeingFlushed(omx_audio_mixer_component_Private)) {
         break;
       }
       if(omx_audio_mixer_component_Private->state == OMX_StateLoaded || omx_audio_mixer_component_Private->state == OMX_StateInvalid) {
@@ -436,7 +470,7 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
       }
     }
 
-    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++){ 
+    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts;i++){
       DEBUG(DEB_LEV_SIMPLE_SEQ, "Waiting for buffer %i semval=%d \n",(int)i,pSem[i]->semval);
       if(pSem[i]->semval>0 && isBufferNeeded[i]==OMX_TRUE && PORT_IS_ENABLED(pPort[i])) {
         tsem_down(pSem[i]);
@@ -452,14 +486,14 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
     }
 
     if(isBufferNeeded[nOutputPortIndex]==OMX_FALSE) {
-       
+
       if(omx_audio_mixer_component_Private->pMark.hMarkTargetComponent != NULL){
         pBuffer[nOutputPortIndex]->hMarkTargetComponent = omx_audio_mixer_component_Private->pMark.hMarkTargetComponent;
         pBuffer[nOutputPortIndex]->pMarkData            = omx_audio_mixer_component_Private->pMark.pMarkData;
         omx_audio_mixer_component_Private->pMark.hMarkTargetComponent = NULL;
         omx_audio_mixer_component_Private->pMark.pMarkData            = NULL;
       }
-      for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++){ 
+      for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++){
         if(isBufferNeeded[i]==OMX_FALSE && PORT_IS_ENABLED(pPort[i])) {
 
           if(isBufferNeeded[i]==OMX_FALSE) {
@@ -482,9 +516,9 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
             pBuffer[nOutputPortIndex]->nTimeStamp = pBuffer[i]->nTimeStamp;
           }
 
-          if(pBuffer[i]->nFlags==OMX_BUFFERFLAG_EOS && pBuffer[i]->nFilledLen==0) {
-            DEBUG(DEB_LEV_FULL_SEQ, "Detected EOS flags in input buffer filled len=%d\n", (int)pBuffer[i]->nFilledLen);
-            //pBuffer[nOutputPortIndex]->nFlags=pBuffer[i]->nFlags;
+          if((pBuffer[i]->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS && pBuffer[i]->nFilledLen==0) {
+            DEBUG(DEB_LEV_FULL_SEQ, "Detected EOS flags in input buffer %p of %i filled len=%d\n", pBuffer[i], (int)i, (int)pBuffer[i]->nFilledLen);
+            pBuffer[nOutputPortIndex]->nFlags = pBuffer[i]->nFlags;
             pBuffer[i]->nFlags=0;
             (*(omx_audio_mixer_component_Private->callbacks->EventHandler))
               (openmaxStandComp,
@@ -494,32 +528,38 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
               pBuffer[nOutputPortIndex]->nFlags, /* The state has been changed in message->messageParam2 */
               NULL);
           }
-          //TBD: Tobe verified
-          if (omx_audio_mixer_component_Private->BufferMgmtCallback && pBuffer[i]->nFilledLen != 0) {
-            (*(omx_audio_mixer_component_Private->BufferMgmtCallback))(openmaxStandComp, pBuffer[i], pBuffer[nOutputPortIndex]);
+
+          //TBD: To be verified
+          if(omx_audio_mixer_component_Private->state == OMX_StateExecuting)  {
+            if (omx_audio_mixer_component_Private->BufferMgmtCallback && pBuffer[i]->nFilledLen != 0) {
+              (*(omx_audio_mixer_component_Private->BufferMgmtCallback))(openmaxStandComp, pBuffer[i], pBuffer[nOutputPortIndex]);
+            } else {
+              /*It no buffer management call back the explicitly consume input buffer*/
+              pBuffer[i]->nFilledLen = 0;
+            }
           } else {
-            /*It no buffer management call back the explicitly consume input buffer*/
-            pBuffer[i]->nFilledLen = 0;
+            DEBUG(DEB_LEV_ERR, "In %s Received Buffer in non-Executing State(%x)\n", __func__, (int)omx_audio_mixer_component_Private->state);
+            if(OMX_TransStateExecutingToIdle == omx_audio_mixer_component_Private->transientState ||
+			   OMX_TransStatePauseToIdle == omx_audio_mixer_component_Private->transientState) {
+              pBuffer[i]->nFilledLen = 0;
+            }
           }
+
           /*Input Buffer has been completely consumed. So, get new input buffer*/
           if(pBuffer[i]->nFilledLen==0) {
             isBufferNeeded[i] = OMX_TRUE;
           }
         }
-      }      
-      
-      if(omx_audio_mixer_component_Private->state==OMX_StatePause && 
-        !(PORT_IS_BEING_FLUSHED(pPort[0]) || 
-          PORT_IS_BEING_FLUSHED(pPort[1]) || 
-          PORT_IS_BEING_FLUSHED(pPort[2]) || 
-          PORT_IS_BEING_FLUSHED(pPort[3]) || 
-          PORT_IS_BEING_FLUSHED(pPort[nOutputPortIndex]))) {
+      }
+
+      if(omx_audio_mixer_component_Private->state==OMX_StatePause &&
+        !(checkAnyPortBeingFlushed(omx_audio_mixer_component_Private))) {
         /*Waiting at paused state*/
         tsem_wait(omx_audio_mixer_component_Private->bStateSem);
       }
 
       /*If EOS and Input buffer Filled Len Zero then Return output buffer immediately*/
-      if(pBuffer[nOutputPortIndex]->nFilledLen!=0 || pBuffer[nOutputPortIndex]->nFlags==OMX_BUFFERFLAG_EOS){
+      if(pBuffer[nOutputPortIndex]->nFilledLen!=0 || (pBuffer[nOutputPortIndex]->nFlags & OMX_BUFFERFLAG_EOS) == OMX_BUFFERFLAG_EOS){
         DEBUG(DEB_LEV_SIMPLE_SEQ, "Returning output buffer \n");
         pPort[nOutputPortIndex]->ReturnBufferFunction(pPort[nOutputPortIndex],pBuffer[nOutputPortIndex]);
         pBuffer[nOutputPortIndex]=NULL;
@@ -529,18 +569,14 @@ void* omx_audio_mixer_BufferMgmtFunction (void* param) {
 
     DEBUG(DEB_LEV_FULL_SEQ, "Input buffer arrived\n");
 
-    if(omx_audio_mixer_component_Private->state==OMX_StatePause && 
-      !(PORT_IS_BEING_FLUSHED(pPort[0]) || 
-        PORT_IS_BEING_FLUSHED(pPort[1]) || 
-        PORT_IS_BEING_FLUSHED(pPort[2]) || 
-        PORT_IS_BEING_FLUSHED(pPort[3]) || 
-        PORT_IS_BEING_FLUSHED(pPort[nOutputPortIndex]))) {
+    if(omx_audio_mixer_component_Private->state==OMX_StatePause &&
+      !(checkAnyPortBeingFlushed(omx_audio_mixer_component_Private))) {
       /*Waiting at paused state*/
       tsem_wait(omx_audio_mixer_component_Private->bStateSem);
     }
 
     /*Input Buffer has been completely consumed. So, return input buffer*/
-    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++){ 
+    for(i=0;i<omx_audio_mixer_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts-1;i++){
       if(isBufferNeeded[i] == OMX_TRUE && pBuffer[i]!=NULL && PORT_IS_ENABLED(pPort[i])) {
         pPort[i]->ReturnBufferFunction(pPort[i],pBuffer[i]);
         pBuffer[i]=NULL;
