@@ -1,11 +1,12 @@
 /**
-  @file src/omx_create_loaders.h
+  src/omx_create_loaders.h
 
   Define a generic entry point for component loaders creation.
-  Each component loader should provide an implementation of the 
+  Each component loader should provide an implementation of the
   createComponentLoaders function.
 
-  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
+  Copyright (C) 2007-2009 STMicroelectronics
+  Copyright (C) 2007-2009 Nokia Corporation and/or its subsidiary(-ies).
 
   This library is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the Free
@@ -22,10 +23,6 @@
   51 Franklin St, Fifth Floor, Boston, MA
   02110-1301  USA
 
-  $Date$
-  Revision $Rev$
-  Author $Author$
-
 */
 
 #ifndef __OMX_CREATE_LOADERS_H__
@@ -35,7 +32,7 @@
 extern "C" {
 #endif
 
-int 
+int
 createComponentLoaders(void);
 
 #ifdef __cplusplus
